string description = "jl: simple .fx demo";

float4 LightPos : Position
<
	string Object = "PointLight";
	string Space = "World";
> = {-10.0f, 10.0f, -10.0f, 0.0f};

float4 SpecularColor : Specular
<
	string UIName = "Specular Color";
> = {0.6f, 1.0f, 0.6f, 1.0f};

float SpecularExponent : SpecularPower
<
	string UIWidget = "Slider";
	float UIMin = 1.0;
	float UIMax = 128.0;
	float UIStep = 1.0;
	string UIName = "Specular Exponent";
> = 30.0;

float4x4 WIT : WorldInverseTranspose;
float4x4 WVP : WorldViewProjection;
float4x4 W : World;
float4x4 VI : ViewInverse;

texture DiffuseTexture : Diffuse
<
	string ResourceName = "texture.dds";
>;

sampler DiffuseMap = sampler_state 
{
	// fr direkte Verwendung in DirectX die nchste Zeile auskommentieren
	//texture = <DiffuseTexture>;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
	AddressW  = CLAMP;
	MIPFILTER = LINEAR;
	MINFILTER = LINEAR;
	MAGFILTER = LINEAR;
};

struct appdata
{
	float3 Position	: POSITION;
	float3 Normal	: NORMAL;
	float2 UV	: TEXCOORD0;
};

struct vertexOutput
{
	float4 HPosition	: POSITION;
	float2 UV		: TEXCOORD0;
	float3 LightVec		: TEXCOORD1;
	float3 WorldNormal	: TEXCOORD2;
	float3 WorldView	: TEXCOORD3;
};

struct pixelOutput
{
	float4 col : COLOR;
};

vertexOutput myVertexShader(appdata IN)
{
	vertexOutput OUT;
	OUT.WorldNormal = normalize(mul(float4(IN.Normal, 0.0), WIT).xyz);
	float4 Po = float4(IN.Position, 1.0);
	float3 Pw = mul(Po, W).xyz;
	OUT.LightVec = normalize(LightPos - Pw);
	OUT.WorldView = normalize(VI[3].xyz - Pw);
	OUT.HPosition = mul(Po, WVP);
	OUT.UV = IN.UV;
	return OUT;
}

pixelOutput myPixelShader(vertexOutput IN)
{
	pixelOutput OUT; 
	float3 Ln = normalize(IN.LightVec);
	float3 Nn = normalize(IN.WorldNormal);
	float ldn = dot(Ln, Nn);
	float3 Vn = normalize(IN.WorldView);
	float3 Hn = normalize(Vn + Ln);
	float hdn = pow(max(0, dot(Hn, Nn)), SpecularExponent);
	OUT.col = hdn*SpecularColor + max(0, ldn)*tex2D(DiffuseMap, IN.UV);
	return OUT;
}

technique dx9
{
	pass p0 
	{		
		VertexShader = compile vs_2_0 myVertexShader();
		ZEnable = true;
		ZWriteEnable = true;
		CullMode = None;
		PixelShader = compile ps_2_0 myPixelShader();
	}
}